/*
 *   ORXONOX - the hottest 3D action shooter ever to exist
 *                    > www.orxonox.net <
 *
 *
 *   License notice:
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *   Author:
 *      Reto Grieder
 *   Co-authors:
 *      ...
 *
 */

/**
@file
@brief
    Implementation of the Game class.
*/

#include "Game.h"

#include <exception>

#include "util/Debug.h"
#include "util/Exception.h"
#include "util/Sleep.h"
#include "Core.h"

namespace orxonox
{
    Game* Game::singletonPtr_s = 0;

    Game::Game(const std::string& cmdLine)
    {
        this->bAbort_ = false;

        // Create the Core
        this->core_.reset(new Core(cmdLine));
    }

    /**
    @brief
        All destruction code is handled by QScopedPointers.
    */
    Game::~Game()
    {
    }

    void Game::setConfigValues()
    {
        /*
        SetConfigValue(statisticsRefreshCycle_, 250000)
            .description("Sets the time in microseconds interval at which average fps, etc. get updated.");
        SetConfigValue(statisticsAvgLength_, 1000000)
            .description("Sets the time in microseconds interval at which average fps, etc. gets calculated.");
        SetConfigValue(fpsLimit_, 50)
            .description("Sets the desired frame rate (0 for no limit).");
        */
    }

    /**
    @brief
        Main loop of the orxonox game.
    */
    void Game::run()
    {
        while (!this->bAbort_)
        {
            try
                { this->core_->update(); }
            catch (...)
            {
                COUT(0) << "An exception occurred in the Core postUpdate: " << Exception::handleMessage() << std::endl;
                COUT(0) << "This should really never happen! Closing the program." << std::endl;
                this->stop();
                break;
            }
        }
    }

    void Game::stop()
    {
        this->bAbort_ = true;
    }
}
