/*
 *   ORXONOX - the hottest 3D action shooter ever to exist
 *                    > www.orxonox.net <
 *
 *
 *   License notice:
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *   Author:
 *      Reto Grieder
 *   Co-authors:
 *      ...
 *
 */

/**
@file
@ingroup Management CoreGame
@brief
    Declaration of Game Singleton which is responsible for running the game.
 */

#ifndef _Game_H__
#define _Game_H__

#include "CorePrereqs.h"

#include <string>
#include <QScopedPointer>
#include "util/Singleton.h"

namespace orxonox
{
    /**
    @brief
        Main class responsible for running the game.
    @remark
        You should only create this singleton once because it owns the Core class! (see remark there)
    */
    class _CoreExport Game : public Singleton<Game>
    {
        friend class Singleton<Game>;

    public:
        Game(const std::string& cmdLine);
        ~Game();

        void setConfigValues();

        void run();
        void stop();

    private:
        Game(Game&); // don't mess with singletons

        QScopedPointer<Core>               core_;
        bool                               bAbort_;

        static Game* singletonPtr_s;        //!< Pointer to the Singleton
    };
}

#endif /* _Game_H__ */
